/*
 * TextEntryList.java
 *
 * Created on February 24, 2004, 11:13 AM
 */

package com.actuate.ExternalText;

import java.util.*;
import java.sql.*;
import java.io.*;

/**
 * This is the main interface to retrieving externalized text fields 
 * from a database through JDBC.
 * The general calling sequence is: <P>
 * TextEntryList() constructor <P>
 * connectDB() to specify the JDBC connection parameters <P>
 * setAttribute() to specify anything that will be substituted
 *     into the SQL string placeholders later on (locale, reportname, etc) <P>
 * addList() to specify all of the text keys to lookup in one shot <P>
 * addStringKey() alternatively to specify a single lookup key that is character based<P>
 * addIntKey alternatively to specify a single lookup key that is a number<P>
 * translate() to specify the generic SQL query and cause a lookup of whatever
 *    keys have been added so far <P>
 * next() to iterate through the retruned resultset rows <P>
 * getString() and getInt() to get the specific returned fields on the current row <P>
 * disconnectDB to close the DBconnection <P>
 *
 * @author  Jim Rutherford, Actuate Corporation
 * @version 1.0
 */
public class TextEntryList {
    
    private String sqlString;
	private String localeSQLString;
    
    private String textList;
    
    private Vector attributes;
    
    private Connection conn;
    private ResultSet rs;
    private Statement stmt;
    
    private String driverName, connectionString, username, password;
   
    
    /** 
     * Creates a new instance of TextEntryList. <P>
     * This also causes the reading in of important properties from a properties file; 
     * ActuateI18NText.properties
     *<P>
     * The properties that need to be in this file are: <P>
     * actuate.externText.JDBCDriverName <P>
     * actuate.externText.JDBCConnectionURL <P>
     * actuate.externText.username <P>
     * actuate.externText.password <P>
     * actuate.externText.SQLString  - This is the generic version of the SQL that is 
     * used to lookup the external text.  When specifying this 
     * input SQL string, there should be placeholder for each attribute that you set using 
     * setAttribute, and a special placeholder of ":_TextList:" to be the placeholder for the
     * list of textkeys that was formed by calls to addStringKey, addIntKey, or addList.

    */
    public TextEntryList() throws FileNotFoundException, IOException {
        // TODO read properties file and get SQL string 
        attributes = new Vector();
        textList = "";
        Properties prop = new Properties();
        prop.load(getClass().getResourceAsStream("actuatei18ntext.properties"));
        driverName = prop.getProperty( "actuate.externText.JDBCDriverName" );
        connectionString = prop.getProperty( "actuate.externText.JDBCConnectionURL" );
        username = prop.getProperty( "actuate.externText.username" );
        password = prop.getProperty( "actuate.externText.password" );
        sqlString = prop.getProperty( "actuate.externText.SQLString" );
		localeSQLString = prop.getProperty( "actuate.externText.localeSQLString" );
    }
    
    /** 
     * Establishes a JDBC connection to the database.
     * @return void
     */
    public void connectDB() throws SQLException, ClassNotFoundException {
            // Load the driver class
            Class.forName(driverName);
            
            // Create a connection through the DriverManager
            conn = DriverManager.getConnection(connectionString , username , password );       
    }
    
    /** 
     * Closes the current DB resultset, statement, and disconnects from the database.
     * @return void
     */
    public void disconnectDB() throws SQLException {
        stmt.close();
        conn.close();
   }
    /** 
     * Adds a single text (label) key.  This method is generally used repetitively to build 
     * up a list of label "keys" that will ultimately be used in the whereclause of the 
     * SQL that is retrieving translated labels.  <P> 
     * A label key should be a unique identifier 
     * for the text of a label to be translated.  Sometimes this database table key field will be 
     * stored in a character (like varchar) format, and sometimes it will be an integer.
     * use this particular method if it stored in character format.  Further processing will 
     * automatically put quotes around each string key that is submitted, and comma delimit them
     * in order to form the final SQL. 
     *<P>
     * Example: addStringKey( "Work Order") will ultimately look up the one label called 
     * "Work Order" in the translation table for the appropriate locale.
     * @param textKeyName A string field to be used as a key in the SQL lookup for translation.
     * @return void
     */
    public void addStringKey(java.lang.String textKeyName) {
        if (textList != "")
            textList +=",";
        textList += "'" + textKeyName + "'";
    }
    
    /** 
     * Adds a single integer (label) key.  This method is generally used repetitively to build 
     * up a list of label "keys" that will ultimately be used in the whereclause of the 
     * SQL that is retrieving translated labels.  <P> 
     * A label key should be a unique identifier 
     * for the text of a label to be translated.  Sometimes this database table key field will be 
     * stored in a character (like varchar) format, and sometimes it will be an integer.
     * use this particular method if it stored in integer format.  Further processing will 
     * automatically put these together in order to form the final SQL. 
     *<P>
     * Example: addIntKey( 1231 ) will ultimately look up the one label called 
     * with key 1231 in the translation table for the appropriate locale.
     * @param intKeyName An integer field to be used as a key in the SQL lookup for translation.
     * @return void
     */
    public void addIntKey(int intKeyName) {
        if (textList != "")
            textList +=",";
        textList += intKeyName;
    }

  
     /** 
     * Adds an entire list of keys.  This method is generally used by itself to build 
     * up a list of label "keys" that will ultimately be used in the whereclause of the 
     * SQL that is retrieving translated labels.  <P> 
     * A label key should be a unique identifier for the text of a label to be translated.  
     *<P>
     * Example: addList( "1231,1232,1233,1234") will ultimately look up these four integer-keyed labels.
     *  in the translation table for the appropriate locale.
     *<P>
     * Example: addList( "\'Work Order\',\'Due Date\',\'Authorized By\'") will ultimately look up 
     * these three test-keyed labels.  The single quote is mandatory for SQL and the backslash
     * escape syntax shown is Java-specific.  Actuate Basic can drop the backslashes.
     * @param MultipleKeyNames A String list of keys to be used as is in the SQL lookup for translation.
     * @return void
     */
    public void addList(java.lang.String MultipleKeyNames) {
        if (textList != "")
            textList +=",";
        textList += MultipleKeyNames;
    }
    
    /** 
     * Sets an attribute for later substitution into the SQL string. <P>
     * This method is generally called repetitively to specify attributes such as 
     * Locale or Language, ReportName, etc.  When specifying the SQL string as a 
     * parameter to the translate method, the attribute names should appear within 
     * colons.  For example: ":Locale:".
     * @param attributeName The name for this attribute, eg. "Locale"
     * @param attributeValue The value for this attribute, eg. "de_DE"
     * @return void
     */
    public void setAttribute(java.lang.String attributeName, java.lang.String attributeValue) {
        QueryAttribute attr = new QueryAttribute(attributeName, attributeValue );
        attributes.add( attr );
    }
    
    /**
     * Provides the generic SQL string with attribute placeholders, and executes 
     * the translation SQL query. <P>
     * This method is called once to execute the translation SQL.  
     * @return void
     */
    public void translate( ) throws SQLException {

        finalizeSqlString();
        stmt = conn.createStatement();
        rs = stmt.executeQuery( sqlString );
    }
    
    /** 
     * Allows iterating through the resultset rows of translated labels. <P>
     * This method is called in a loop to increment the internal cursor that is kept 
     * on the resulting rows of translated labels and returns true while there are 
     * still rows to process.  For each of these method executions, you should use 
     * getString() and getInt() methods to get the specific fields from the query.
     * @return true if there's another row and false if past last row.
     */
    public boolean next() throws SQLException {
        boolean result = rs.next();
        return result;
    }
	
	public void getDefaultLocale() throws SQLException {
		
		stmt = conn.createStatement();
        rs = stmt.executeQuery( localeSQLString );
	}
    
    /** 
     * Gets a specific string value back from the current resultset row. <P>
     * This method is generally called repetitively (and with getInt) to get 
     * the individual fields back from the resultset row.  An example call 
     * might be: getString("TranslatedLabel"); or getString("FontFaceName");
     * @param colName The name of the column to get from this resultset row.  
     * This must correspond exactly (case insensitive) with a column that is 
     * being returned by the SQL statement.
     * @return String value of this field for the current resultset row.
     */
    public String getString ( String colName ) throws SQLException {
        String result = rs.getString( colName );
        return result;
    }
  
    /** 
     * Gets a specific integer value back from the current resultset row.
     * <P>
     * This method is generally called repetitively (and with getString) to get 
     * the individual fields back from the resultset row.  An example call 
     * might be: getInt("FieldWidth"); or getInt("FontSize");
     * @param colName The name of the column to get from this resultset row.  
     * This must correspond exactly (case insensitive) with a column that is 
     * being returned by the SQL statement.
     * @return Integer value of this field for the current resultset row.
     */
    public int getInt ( String colName ) throws SQLException {
        int result = rs.getInt( colName );
        return result;
    }
    
    private void finalizeSqlString(){
        // perform substitution for each attribute
        //perform substitution on list by making it an attribute forst
        setAttribute("_TextList" , textList );
        
        QueryAttribute thisAttr;
        Enumeration attrEnum = attributes.elements();
        
        while( attrEnum.hasMoreElements()) {
            thisAttr = (QueryAttribute) attrEnum.nextElement();
            String targetString = ":" + thisAttr.getAttrName() + ":";
            int targetLength = targetString.length();
            int substIndex = sqlString.indexOf( targetString );
            if ( substIndex >= 0 ) {
                String beginString = sqlString.substring(0,substIndex);
                String endString = sqlString.substring( substIndex + targetLength);
                sqlString = beginString + thisAttr.equals() + endString ; 
            }
        }
    }
}
